"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Destination = exports.fieldsToJsonSchema = exports.hookTypeStrings = void 0;
const destination_subscriptions_1 = require("@segment/destination-subscriptions");
const action_1 = require("./action");
Object.defineProperty(exports, "hookTypeStrings", { enumerable: true, get: function () { return action_1.hookTypeStrings; } });
const time_1 = require("../time");
const fields_to_jsonschema_1 = require("./fields-to-jsonschema");
Object.defineProperty(exports, "fieldsToJsonSchema", { enumerable: true, get: function () { return fields_to_jsonschema_1.fieldsToJsonSchema; } });
const create_request_client_1 = __importDefault(require("../create-request-client"));
const schema_validation_1 = require("../schema-validation");
const errors_1 = require("../errors");
const parse_settings_1 = require("./parse-settings");
const retry_1 = require("../retry");
const instanceOfAudienceDestinationSettingsWithCreateGet = (object) => {
    return 'createAudience' in object && 'getAudience' in object;
};
class Destination {
    constructor(destination) {
        this.definition = destination;
        this.name = destination.name;
        this.extendRequest = destination.extendRequest;
        this.actions = {};
        this.authentication = destination.authentication;
        this.responses = [];
        if (this.definition.onDelete) {
            this.onDelete = this._onDelete;
        }
        if (this.authentication?.fields) {
            this.settingsSchema = fields_to_jsonschema_1.fieldsToJsonSchema(this.authentication.fields);
        }
        for (const action of Object.keys(destination.actions)) {
            this.partnerAction(action, destination.actions[action]);
        }
    }
    validateSettings(settings) {
        if (this.settingsSchema) {
            try {
                schema_validation_1.validateSchema(settings, this.settingsSchema, { schemaKey: `${this.name}:settings` });
            }
            catch (err) {
                const error = err;
                if (error.name === 'AggregateAjvError' || error.name === 'ValidationError') {
                    error.status = 400;
                }
                throw error;
            }
        }
    }
    async createAudience(createAudienceInput) {
        const audienceDefinition = this.definition;
        if (!instanceOfAudienceDestinationSettingsWithCreateGet(audienceDefinition.audienceConfig)) {
            throw new Error('Unexpected call to createAudience');
        }
        const destinationSettings = this.getDestinationSettings(createAudienceInput.settings);
        const auth = parse_settings_1.getAuthData(createAudienceInput.settings);
        const context = {
            audienceSettings: createAudienceInput.audienceSettings,
            settings: destinationSettings,
            payload: undefined,
            auth
        };
        const options = this.extendRequest?.(context) ?? {};
        const requestClient = create_request_client_1.default({ ...options, statsContext: context.statsContext });
        return audienceDefinition.audienceConfig?.createAudience(requestClient, createAudienceInput);
    }
    async getAudience(getAudienceInput) {
        const audienceDefinition = this.definition;
        if (!instanceOfAudienceDestinationSettingsWithCreateGet(audienceDefinition.audienceConfig)) {
            throw new Error('Unexpected call to getAudience');
        }
        const destinationSettings = this.getDestinationSettings(getAudienceInput.settings);
        const auth = parse_settings_1.getAuthData(getAudienceInput.settings);
        const context = {
            audienceSettings: getAudienceInput.audienceSettings,
            settings: destinationSettings,
            payload: undefined,
            auth
        };
        const options = this.extendRequest?.(context) ?? {};
        const requestClient = create_request_client_1.default({ ...options, statsContext: context.statsContext });
        return audienceDefinition.audienceConfig?.getAudience(requestClient, getAudienceInput);
    }
    async testAuthentication(settings) {
        const destinationSettings = this.getDestinationSettings(settings);
        const auth = parse_settings_1.getAuthData(settings);
        const data = { settings: destinationSettings, auth };
        const context = {
            settings: destinationSettings,
            payload: undefined,
            auth
        };
        this.validateSettings(destinationSettings);
        if (!this.authentication?.testAuthentication) {
            return;
        }
        const options = this.extendRequest?.(context) ?? {};
        const requestClient = create_request_client_1.default({ ...options, statsContext: context.statsContext });
        try {
            await this.authentication.testAuthentication(requestClient, data);
        }
        catch (error) {
            const statusCode = error?.response?.status ?? '';
            throw new Error(`Credentials are invalid: ${statusCode} ${error.message}`);
        }
    }
    async refreshAccessToken(settings, oauthData, synchronizeRefreshAccessToken) {
        if (!(this.authentication?.scheme === 'oauth2' || this.authentication?.scheme === 'oauth-managed')) {
            throw new errors_1.IntegrationError('refreshAccessToken is only valid with oauth2 authentication scheme', 'NotImplemented', 501);
        }
        const context = {
            settings,
            payload: undefined,
            auth: parse_settings_1.getAuthData(settings)
        };
        const options = this.extendRequest?.(context) ?? {};
        const requestClient = create_request_client_1.default({ ...options, statsContext: context.statsContext });
        if (!this.authentication?.refreshAccessToken) {
            return undefined;
        }
        await synchronizeRefreshAccessToken?.();
        return this.authentication.refreshAccessToken(requestClient, { settings, auth: oauthData });
    }
    partnerAction(slug, definition) {
        const action = new action_1.Action(this.name, definition, this.extendRequest);
        action.on('response', (response) => {
            if (response) {
                this.responses.push(response);
            }
        });
        this.actions[slug] = action;
        return this;
    }
    async executeAction(actionSlug, { event, mapping, settings, auth, features, statsContext, logger, dataFeedCache, transactionContext, stateContext }) {
        const action = this.actions[actionSlug];
        if (!action) {
            return [];
        }
        let audienceSettings = {};
        if (event.context?.personas) {
            audienceSettings = event.context?.personas?.audience_settings;
        }
        return action.execute({
            mapping,
            data: event,
            settings,
            audienceSettings,
            auth,
            features,
            statsContext,
            logger,
            dataFeedCache,
            transactionContext,
            stateContext
        });
    }
    async executeBatch(actionSlug, { events, mapping, settings, auth, features, statsContext, logger, dataFeedCache, transactionContext, stateContext }) {
        const action = this.actions[actionSlug];
        if (!action) {
            return [];
        }
        let audienceSettings = {};
        if (events[0].context?.personas) {
            audienceSettings = events[0].context?.personas?.audience_settings;
        }
        return action.executeBatch({
            mapping,
            data: events,
            settings,
            audienceSettings,
            auth,
            features,
            statsContext,
            logger,
            dataFeedCache,
            transactionContext,
            stateContext
        });
    }
    async executeDynamicField(actionSlug, fieldKey, data, dynamicFn) {
        const action = this.actions[actionSlug];
        if (!action) {
            return [];
        }
        return action.executeDynamicField(fieldKey, data, dynamicFn);
    }
    async onSubscription(subscription, events, settings, auth, options) {
        const subscriptionStartedAt = time_1.time();
        const actionSlug = subscription.partnerAction;
        const input = {
            mapping: subscription.mapping || {},
            settings,
            auth,
            features: options?.features || {},
            statsContext: options?.statsContext || {},
            logger: options?.logger,
            dataFeedCache: options?.dataFeedCache,
            transactionContext: options?.transactionContext,
            stateContext: options?.stateContext
        };
        let results = null;
        try {
            if (!subscription.subscribe || typeof subscription.subscribe !== 'string') {
                results = [{ output: 'invalid subscription' }];
                return results;
            }
            const parsedSubscription = destination_subscriptions_1.parseFql(subscription.subscribe);
            if (parsedSubscription.error) {
                results = [{ output: `invalid subscription : ${parsedSubscription.error.message}` }];
                return results;
            }
            const isBatch = Array.isArray(events);
            const allEvents = (isBatch ? events : [events]);
            const subscribedEvents = allEvents.filter((event) => destination_subscriptions_1.validate(parsedSubscription, event));
            if (subscribedEvents.length === 0) {
                results = [{ output: 'not subscribed' }];
                return results;
            }
            else if (isBatch) {
                return await this.executeBatch(actionSlug, { ...input, events: subscribedEvents });
            }
            else {
                return await this.executeAction(actionSlug, { ...input, event: subscribedEvents[0] });
            }
        }
        catch (err) {
            const error = err;
            results = [{ error: { message: error.message } }];
            if (error.name === 'AggregateAjvError' || error.name === 'ValidationError') {
                error.status = 400;
            }
            throw error;
        }
        finally {
            const subscriptionEndedAt = time_1.time();
            const subscriptionDuration = time_1.duration(subscriptionStartedAt, subscriptionEndedAt);
            options?.onComplete?.({
                duration: subscriptionDuration,
                destination: this.name,
                action: actionSlug,
                subscribe: subscription.subscribe,
                input: {
                    data: events,
                    mapping: input.mapping,
                    settings: input.settings
                },
                output: results
            });
        }
    }
    onEvent(event, settings, options) {
        return this.onSubscriptions(event, settings, options);
    }
    onBatch(events, settings, options) {
        return this.onSubscriptions(events, settings, options);
    }
    async _onDelete(event, settings, options) {
        const { userId, anonymousId } = event;
        const payload = { userId, anonymousId };
        const destinationSettings = this.getDestinationSettings(settings);
        this.validateSettings(destinationSettings);
        const auth = parse_settings_1.getAuthData(settings);
        const data = {
            payload,
            settings: destinationSettings,
            auth
        };
        const context = {
            settings: destinationSettings,
            payload,
            auth
        };
        const opts = this.extendRequest?.(context) ?? {};
        const requestClient = create_request_client_1.default({ ...opts, statsContext: context.statsContext });
        const run = async () => {
            const deleteResult = await this.definition.onDelete?.(requestClient, data);
            const result = deleteResult ?? { output: 'no onDelete defined' };
            return result;
        };
        const onFailedAttempt = async (error) => {
            const statusCode = error?.status ?? error?.response?.status ?? 500;
            if (!(statusCode === 401 &&
                (this.authentication?.scheme === 'oauth2' || this.authentication?.scheme === 'oauth-managed'))) {
                throw error;
            }
            const oauthSettings = parse_settings_1.getOAuth2Data(settings);
            const newTokens = await this.refreshAccessToken(destinationSettings, oauthSettings, options?.synchronizeRefreshAccessToken);
            if (!newTokens) {
                throw new errors_1.InvalidAuthenticationError('Failed to refresh access token', errors_1.ErrorCodes.OAUTH_REFRESH_FAILED);
            }
            settings = parse_settings_1.updateOAuthSettings(settings, newTokens);
            await options?.onTokenRefresh?.(newTokens);
        };
        return await retry_1.retry(run, { retries: 2, onFailedAttempt });
    }
    async onSubscriptions(data, settings, options) {
        const subscriptions = this.getSubscriptions(settings);
        const destinationSettings = this.getDestinationSettings(settings);
        this.validateSettings(destinationSettings);
        const run = async () => {
            const authData = parse_settings_1.getAuthData(settings);
            const promises = subscriptions.map((subscription) => this.onSubscription(subscription, data, destinationSettings, authData, options));
            const results = await Promise.all(promises);
            return [].concat(...results);
        };
        const onFailedAttempt = async (error) => {
            const statusCode = error?.status ?? error?.response?.status ?? 500;
            if (!(statusCode === 401 &&
                (this.authentication?.scheme === 'oauth2' || this.authentication?.scheme === 'oauth-managed'))) {
                throw error;
            }
            const oauthSettings = parse_settings_1.getOAuth2Data(settings);
            const newTokens = await this.refreshAccessToken(destinationSettings, oauthSettings, options?.synchronizeRefreshAccessToken);
            if (!newTokens) {
                throw new errors_1.InvalidAuthenticationError('Failed to refresh access token', errors_1.ErrorCodes.OAUTH_REFRESH_FAILED);
            }
            settings = parse_settings_1.updateOAuthSettings(settings, newTokens);
            await options?.onTokenRefresh?.(newTokens);
        };
        return await retry_1.retry(run, { retries: 2, onFailedAttempt });
    }
    getSubscriptions(settings) {
        const { subscription, subscriptions } = settings;
        let parsedSubscriptions;
        if (subscription) {
            parsedSubscriptions = [subscription];
        }
        else if (Array.isArray(subscriptions)) {
            parsedSubscriptions = subscriptions;
        }
        else {
            parsedSubscriptions = [];
        }
        return parsedSubscriptions;
    }
    getDestinationSettings(settings) {
        const { subcription, subscriptions, oauth, ...otherSettings } = settings;
        return otherSettings;
    }
}
exports.Destination = Destination;
//# sourceMappingURL=index.js.map